#!/usr/bin/env python

from konfluence import Konfluence
from pylab import *
from tinkt.klasses import *

kfl = Konfluence()
kfl.register('colormap', ColorMap, generator='generate')
kfl.register('colornorm', ColorNorm, generator='generate')
kfl.register('colorspec', ColorSpec)

spec = kfl.get('colorspec/test3')

print spec.cmap, spec.norm

data = np.random.rand(12, 20) * 2500

spec.cmap.set_under('none')
plt.pcolormesh(data, cmap=spec.cmap, norm=spec.norm)
plt.colorbar()

plt.show()


# -*- coding:utf-8 -*-

# cmap utils

import six

import numpy as np
from matplotlib import cm as mpl_cm
from matplotlib import colors as mpl_colors
from . import cm as tinkt_cm


CM_FAMILIES = {
    'mpl': mpl_cm,
    'tinkt': tinkt_cm
}


def set_under_over_bad_colors(cmap, under=None, over=None, bad=None):
    if under is not None:
        cmap.set_under(under)
    if over is not None:
        cmap.set_over(over)
    if bad is not None:
        cmap.set_bad(bad)
    return cmap


def get_cmap(base_cmap,
             clip_min=None, clip_max=None,
             N=None,
             sample_points=None,
             bad=None, over=None, under=None,
             *args, **kwargs):
    """
    Get cmap object by name, and optionally tweak it into a new one.
    Currently only supports tweaking of continuous cmaps.
    :param base_cmap: either a name or a cmap object.
    :param clip_min: lower clip point, valid range: 0.0~1.0, default: None.
    :param clip_max: upper clip point, valid range: 0.0~1.0, default: None.
    :param N: new cmap's color number, default: None (inherits from base_cmap).
    :param sample_points: a series of sampling points (0.0~1.0) on the base_cmap. When using this arg, clip_min, clip_max and N are ignored.
    :param bad: bad color, default None (inherits from base_cmap)
    :param over: over color, default None (inherits from base_cmap)
    :param under: under color, default None (inherits from base_cmap)
    :return: a cmap object (matplotlib.colors.Colormap)
    """

    if isinstance(base_cmap, tuple):
        # The tuple-form is for compatibility of old codes using metlib.color.cmap_utils.get_cmap , which read opts from json file.
        # Please neglect the complex logics and use named args whenever possible.
        return _parse_tuple_form_args_for_get_cmap(base_cmap)

    if isinstance(base_cmap, six.string_types):
        for cm_family in CM_FAMILIES.values():
            try:
                base_cmap = getattr(cm_family, base_cmap)
                break
            except AttributeError:
                pass

    if not isinstance(base_cmap, mpl_colors.Colormap):
        raise RuntimeError(u'Cannot find base_cmap: {}'.format(base_cmap))

    if sample_points is not None:
        new_name = u'Resampled from {}'.format(base_cmap.name)
        new_cmap = mpl_colors.LinearSegmentedColormap.from_list(new_name, base_cmap(sample_points))
    elif clip_min is not None or clip_max is not None:
        clip_min = 0.0 if clip_min is None else float(clip_min)
        clip_max = 0.0 if clip_max is None else float(clip_max)
        N = base_cmap.N if N is None else int(N)
        sample_points = np.linspace(clip_min, clip_max, N)
        new_name = u'Clipped from {}'.format(base_cmap.name)
        new_cmap = mpl_colors.LinearSegmentedColormap.from_list(new_name, base_cmap(sample_points))
    else:
        N = int(N) if N is not None else base_cmap.N
        new_cmap = base_cmap._resample(N)

    if bad is not None:
        new_cmap.set_bad(bad)
    elif base_cmap._rgba_bad:
        new_cmap.set_bad(base_cmap._rgba_bad)

    if over is not None:
        new_cmap.set_over(over)
    elif base_cmap._rgba_over:
        new_cmap.set_over(base_cmap._rgba_over)

    if under is not None:
        new_cmap.set_under(under)
    elif base_cmap._rgba_under:
        new_cmap.set_under(base_cmap._rgba_under)

    return new_cmap


def _parse_tuple_form_args_for_get_cmap(opts):
    # The tuple-form is for compatibility of old codes using metlib.color.cmap_utils.get_cmap, which read opts from json file.
    if len(opts) == 1:
        return get_cmap(opts[0])
    elif len(opts) == 2:
        if isinstance(opts[1], (tuple, list, np.ndarray)):
            if len(opts[1]) == 0:
                return get_cmap(opts[0])
            elif len(opts[1]) == 1:
                if isinstance(opts[1][0], (tuple, list, np.ndarray)):
                    return get_cmap(opts[0], sample_points=opts[1][0])
                else:
                    raise ValueError("")
            elif len(opts[1]) == 2:
                clip_min, clip_max = opts[1]
                N = None
            elif len(opts[1]) == 3:
                clip_min, clip_max, N = opts[1]
            else:
                return get_cmap(opts[0], sample_points=opts[1])
            return get_cmap(opts[0], clip_min=clip_min, clip_max=clip_max, N=N)
        else:
            raise ValueError("")
    else:
        raise ValueError("")



# -*- coding:utf-8 -*-

import six
from konfluence import Konfluence
import numpy as np
from matplotlib import colors as mpl_colors
from matplotlib import pyplot as plt


class ColorSpec(object):
    konf = Konfluence()

    def __init__(self, name, cmap, norm=None, categories=None, label=None):
        self.name = name
        try:
            if isinstance(cmap, mpl_colors.Colormap):
                self.cmap = cmap
            elif isinstance(cmap, six.string_types):
                self.cmap = self.konf.get(cmap, 'default')
            elif isinstance(cmap, (list, tuple, np.ndarray)):
                self.cmap = mpl_colors.ListedColormap(cmap)
            else:
                raise ValueError
        except Exception as e:
            raise ValueError(u'Cannot parse cmap: {}'.format(cmap))

        try:
            if norm is None or isinstance(norm, mpl_colors.Normalize):
                self.norm = norm
            elif isinstance(norm, six.string_types):
                self.norm = self.konf[norm]
            elif isinstance(norm, (list, tuple, np.ndarray)):
                self.norm = mpl_colors.BoundaryNorm(norm, len(norm)-1, clip=False)
            else:
                raise ValueError
        except Exception as e:
            raise ValueError(u'Cannot parse norm: {}'.format(norm))

        self.categories = categories

        self.label = label

    def colorbar(self, **kwargs):
        kwargs.update({})
        clb = plt.colorbar(**kwargs)
        if self.label:
            clb.set_label(self.label)
        return clb




# -*- coding:utf-8 -*-

import six
from matplotlib import colors as mpl_colors
from .. import cmap_utils


class ColorMap(object):
    def __init__(self, name,
                 type='Normal',
                 base_cmap_name=None,
                 clip_min=None, clip_max=None,
                 N=None,
                 sample_points=None,
                 colors=None,
                 bad=None, over=None, under=None
                 ):
        self.name = name
        self.type = type
        if self.type == 'Normal':
            self.base_cmap = cmap_utils.get_cmap(base_cmap_name,
                                                       clip_min=clip_min, clip_max=clip_max,
                                                       N=N,
                                                       sample_points=sample_points,
                                                       bad=bad, over=over, under=under)
        elif self.type == 'Listed':
            if colors:
                self.base_cmap = mpl_colors.ListedColormap(colors, name=self.name, N=N)
                cmap_utils.set_under_over_bad_colors(self.base_cmap, under=under, over=over, bad=bad)
            else:
                raise NotImplementedError()

    def generate(self, *args):
        if self.type == 'Normal':
            return self._gen_normal(*args)
        elif self.type == 'Listed':
            return self._gen_listed(*args)

    def _gen_normal(self, clip_min=None, clip_max=None, N=None, *args, **kwargs):
        return cmap_utils.get_cmap(self.base_cmap, clip_min=clip_min, clip_max=clip_max, N=N)

    def _gen_listed(self, *args):
        # TODO: implement
        return self.base_cmap




# -*- coding:utf-8 -*-

import setuptools

with open("README.md", "r") as fh:
    long_description = fh.read()


setuptools.setup(
    name="tinkt",
    version="0.0.6",
    author="claydodo and his little friends (xiao huo ban)",
    author_email="claydodo@foxmail.com",
    description="color and colormap utils (for matplotlib)",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/claydodo/tinkt",
    packages=setuptools.find_packages(),
    classifiers=(
        "Programming Language :: Python :: 2.7 ",
        "Operating System :: OS Independent",
    ),
    install_requires=[
        'konfluence >= 0.0.1'
    ]
)



# -*- coding:utf-8 -*-

import six
import numpy as np
from numpy import ma
from matplotlib import colors as mpl_colors


class DecreasableBoundaryNorm(mpl_colors.BoundaryNorm):
    """
    Generate a colormap index based on discrete intervals.

    Unlike :class:`Normalize` or :class:`LogNorm`,
    :class:`BoundaryNorm` maps values to integers instead of to the
    interval 0-1.

    Mapping to the 0-1 interval could have been done via
    piece-wise linear interpolation, but using integers seems
    simpler, and reduces the number of conversions back and forth
    between integer and floating point.
    """
    def __init__(self, boundaries, ncolors, clip=False):
        """
        *boundaries*
            a monotonically increasing/decreasing sequence
        *ncolors*
            number of colors in the colormap to be used

        If::

            b[i] <= v < b[i+1]

        then v is mapped to color j;
        as i varies from 0 to len(boundaries)-2,
        j goes from 0 to ncolors-1.

        Out-of-range values are mapped
        to -1 if low and ncolors if high; these are converted
        to valid indices by
        :meth:`Colormap.__call__` .
        If clip == True, out-of-range values
        are mapped to 0 if low and ncolors-1 if high.
        """
        self.clip = clip
        if boundaries[0] <= boundaries[-1]:
            self.increasing = True
            self.vmin = boundaries[0]
            self.vmax = boundaries[-1]
        else:
            self.increasing = False
            self.vmin = boundaries[-1]
            self.vmax = boundaries[0]

        self.boundaries = np.asarray(boundaries)
        self.N = len(self.boundaries)
        self.Ncmap = ncolors
        if self.N - 1 == self.Ncmap:
            self._interp = False
        else:
            self._interp = True

    def __call__(self, value, clip=None):
        if clip is None:
            clip = self.clip

        xx, is_scalar = self.process_value(value)
        mask = ma.getmaskarray(xx)
        xx = np.atleast_1d(xx.filled(self.vmax + 1))
        if clip:
            np.clip(xx, self.vmin, self.vmax, out=xx)
            max_col = self.Ncmap - 1
        else:
            max_col = self.Ncmap
        iret = np.zeros(xx.shape, dtype=np.int16)
        for i, b in enumerate(self.boundaries):
            if self.increasing:
                iret[xx >= b] = i
            else:
                iret[xx <= b] = i
        if self._interp:
            scalefac = float(self.Ncmap - 1) / (self.N - 2)
            iret = (iret * scalefac).astype(np.int16)
        if self.increasing:
            iret[xx < self.vmin] = -1
            iret[xx >= self.vmax] = max_col
        else:
            iret[xx > self.vmax] = -1
            iret[xx <= self.vmin] = max_col
        ret = ma.array(iret, mask=mask)
        if is_scalar:
            ret = int(ret[0])  # assume python scalar
        return ret


class ColorNorm(object):
    def __init__(self, name,
                 type, para):
        self.name = name
        self.type = type

        self.norm = None
        if self.type == 'BoundaryNorm':
            if isinstance(para, (list, tuple, np.ndarray)):
                self.norm = mpl_colors.BoundaryNorm(para, len(para)-1)
            elif isinstance(para, dict):
                self.norm = mpl_colors.BoundaryNorm(**para)
            else:
                raise ValueError("Bad para for BoundaryNorm: {}".format(para))

    def generate(self, *args):
        return self.norm